﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Timers;
using System.Threading;
using System.Threading.Tasks;
using System.IO;
using System.Text.RegularExpressions;
using Raspberry.IO.GeneralPurpose;

namespace Scheduler
{
    class Program
    {
        System.Timers.Timer checkSchedulerTimer;
        List<Schedule> scheduleList = new List<Schedule>();
        const string SCHEDULE_FILE_PATH = @"/home/pi/scheduler/schedule.txt";

        const ConnectorPin pin4 = ConnectorPin.P1Pin07;
        const ConnectorPin pin17 = ConnectorPin.P1Pin11;
        const ConnectorPin pin18 = ConnectorPin.P1Pin12;

        const string HEATUP = "HEATUP";
        const string HEATDOWN = "HEATDOWN";

        static void Main(string[] args)
        {
            Program instance = new Program();
            if (args.Length == 0)
            {
                instance.InitializeMotor();
                instance.LoadSchedule();
                instance.RunScheduler();
            }
            else
            {
                if (args.Length > 1)
                {
                    Console.WriteLine("Incorrect Arguement. You may run the scheduler with following arguments:");
                    Console.WriteLine("sudo mono Scheduler.exe Forward");
                    Console.WriteLine("sudo mono Scheduler.exe Reverse");
                    Console.WriteLine("sudo mono Scheduler.exe Schedule");
                    Environment.Exit(1);
                }
                else
                {
                    if (args[0].Equals("Forward"))
                    {
                        instance.InitializeMotor();
                        instance.HeatDown();
                    }
                    if (args[0].Equals("Reverse"))
                    {
                        instance.InitializeMotor();
                        instance.HeatUp();
                    }
                    if (args[0].Equals("Schedule"))
                    {
                        instance.GetScheduleFromFile();
                    }
                }
            }
        }

        private void RunScheduler()
        {
            checkSchedulerTimer = new System.Timers.Timer(60000); // 60 seconds
            checkSchedulerTimer.Enabled = true;
            int currentYear = DateTime.Now.Year;
            int currentMonth = DateTime.Now.Month;
            int currentDay = DateTime.Now.Day;
            DateTime todaysDate = new DateTime(currentYear, currentMonth, currentDay);
            checkSchedulerTimer.Elapsed += new ElapsedEventHandler(CheckSchedule);

            try
            {
                while (true)
                {
                    // run in infinite loop
                    // the timer will fire every minute to check if its time to run the scheduled action
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
                Logger.LogError(DateTime.Now.ToString(), e.Message);
                Environment.Exit(1);
            }
        }

        private void CheckSchedule(object source, ElapsedEventArgs e)
        {
            string currentDateTime = DateTime.Now.ToString("h:mm tt");
            foreach (Schedule scheduleItem in scheduleList)
            {
                if (currentDateTime == scheduleItem.scheduleTime && scheduleItem.scheduleCommand == HEATUP)
                {
                    HeatUp();
                    Logger.LogScheduleStats(DateTime.Now.ToString(), HEATUP);
                }
                if (currentDateTime == scheduleItem.scheduleTime && scheduleItem.scheduleCommand == HEATDOWN)
                {
                    HeatDown();
                    Logger.LogScheduleStats(DateTime.Now.ToString(), HEATDOWN);
                }
            }
        }

        private void InitializeMotor()
        {
            pin4.Output().Name("PIN4").Enable();
            pin17.Output().Name("PIN17").Enable();
            Console.WriteLine("PIN4, PIN17 initialized as OUTPUT");
            Console.WriteLine();
        }

        private void HeatUp()
        {
            var driver = GpioConnectionSettings.DefaultDriver;
            driver.Out(pin4).Write(true);
            driver.Out(pin17).Write(false);
            Thread.Sleep(2000); // 2 seconds
            driver.Out(pin4).Write(false);
            driver.Out(pin17).Write(false);
        }

        private void HeatDown()
        {
            var driver = GpioConnectionSettings.DefaultDriver;
            driver.Out(pin4).Write(false);
            driver.Out(pin17).Write(true);
            Thread.Sleep(2000); // 2 seconds 
            driver.Out(pin4).Write(false);
            driver.Out(pin17).Write(false);
        }

        private void LoadSchedule()
        {
            string line = string.Empty;
            int lineCount = 0;

            Console.WriteLine("Loading schedule from file...");
            Console.WriteLine();

            try
            {
                using (StreamReader schedule = new StreamReader(SCHEDULE_FILE_PATH))
                {
                    while ((line = schedule.ReadLine()) != null)
                    {
                        string[] lineArray = Regex.Split(line, ",");
                        scheduleList.Add(new Schedule { scheduleTime = lineArray[0].Trim(), scheduleCommand = lineArray[1].Trim() });
                        lineCount++;
                    }
                    if (lineCount >= 1)
                    { 
                        Console.WriteLine(lineCount + " scheduled items loaded");
                        Console.WriteLine();
                    }
                    else if (lineCount < 1)
                    {
                        Console.WriteLine("Nothing to schedule, exiting...");
                        Environment.Exit(0);
                    }

                    Console.WriteLine("**************");
                    Console.WriteLine("***SCHEDULE***");
                    Console.WriteLine("**************");

                    foreach (Schedule item in scheduleList)
                    {
                        Console.WriteLine(item.scheduleTime);
                        Console.WriteLine(item.scheduleCommand);
                    }
                }
            }
            catch (Exception e)
            {
                Logger.LogError(DateTime.Now.ToString(), e.Message);
                ScheduleErrorMessage(e.Message);
            }
        }

        private void GetScheduleFromFile()
        {
            string line = string.Empty;
            List<Schedule> scheduledItems = new List<Schedule>();
            try
            {
                using (StreamReader schedule = new StreamReader(SCHEDULE_FILE_PATH))
                {
                    while ((line = schedule.ReadLine()) != null)
                    {
                        string[] lineArray = Regex.Split(line, ",");
                        Console.WriteLine(lineArray[0].Trim());
                        Console.WriteLine(lineArray[1].Trim());
                    }
                }
            }
            catch (Exception e)
            {
                ScheduleErrorMessage(e.Message);
            }
        }

        private void ScheduleErrorMessage(string error)
        {
            Console.WriteLine("The schedule file could not be read:");
            Console.WriteLine();
            Console.WriteLine(error);
            Console.WriteLine();
            Console.WriteLine("Did you remember to create a schedule text file in " + SCHEDULE_FILE_PATH + "?");
            Console.WriteLine();
            Console.WriteLine("The file should be formatted with the Schedule Time followed by a comma, then the Schedule Command: \r\n");
            Console.WriteLine("6:00 AM, HEATUP");
            Console.WriteLine("6:20 AM, HEATDOWN");
            Console.WriteLine("8:00 PM, HEATUP");
            Console.WriteLine("8:20 PM, HEATDOWN");
            Environment.Exit(1);
        }
    }
}
